/*
 * See the files COPYING and README for copyright information and how to reach 
 * the author.
 *
 *  $Id: jobs.c,v 1.3 2004/12/02 15:02:52 lordjaxom Exp $
 */

#include "i18n.h"
#include "jobs.h"
#include "setup.h"
#include "process-dvd.h"
#include "process-vdr.h"
#include <vdr/recording.h>
#include <vdr/interface.h>
#include <vdr/videodir.h>

#define NEXT_ARCHIVE_MARKER "/.nextArchiveDVD"

// --- cBurnRecording ---------------------------------------------------------

cBurnRecording::cBurnRecording(cRecording *Recording, time_t Timestamp) {
	char *ptr, *tmp;
	mFileName = strdup(Recording->FileName());
	strncpy(mName, Recording->Name(), JOBNAMELEN);
	mSummary = (Recording->Summary() ? strdup(Recording->Summary()) : NULL);
	mTempDir = NULL;
	mTimestamp = Timestamp;
	mNumber = -1;
	mSizeMB = 0;

	tmp = mName;
	do {
		ptr = tmp;
		while (strchr("@%", *ptr) != NULL)
			++ptr;
		memmove(tmp, ptr, strlen(ptr) + 1);
		if ((tmp = strchr(tmp, '~')) != NULL)
			++tmp;
	} while (tmp != NULL);

	mTitle = strdup(Recording->Title('\t'));
	ptr = mTitle;
	while ((tmp = strchr(ptr + 1, '\t')) != NULL) {
		*tmp = ' ';
		ptr = tmp;
	}
	*ptr = '\0';
}

cBurnRecording::cBurnRecording(const cBurnRecording &Recording) {
	mFileName = strdup(Recording.mFileName);
	strncpy(mName, Recording.mName, JOBNAMELEN);
	mSummary = (Recording.mSummary ? strdup(Recording.mSummary) : NULL);
	mTitle = strdup(Recording.mTitle);
	mTempDir = (Recording.mTempDir ? strdup(Recording.mTempDir) : NULL);
	mTimestamp = Recording.mTimestamp;
	mNumber = Recording.mNumber;
	mSizeMB = Recording.mSizeMB;
}

cBurnRecording::~cBurnRecording(void) {
	free(mFileName);
	free(mSummary);
	free(mTitle);
	free(mTempDir);
}

void cBurnRecording::SetTitle(const char *Title) {
	free(mTitle);
	mTitle = strdup(Title);
}

void cBurnRecording::SetTempDir(const char *TempDir) {
	free(mTempDir);
	asprintf(&mTempDir, "%s/VDRSYNC.%d", TempDir, mNumber);
}

double cBurnRecording::SizeMB(eDiskType dt) {
	if (mSizeMB == 0)
		mSizeMB = ScanRecordingSize(mFileName, dt) / 1024.f;
	return mSizeMB;
}

const tTrackInfo *cBurnRecording::GetTracks(void) {
	if (mTracks[0].Type == ttUnknown) {
		const tTrackInfo *tracks = FindTitleTracks(mTempDir);
		if (tracks == NULL)
			return NULL;
		memcpy(mTracks, tracks, sizeof(tTrackInfo) * MAXTRACKS);
	}
	return mTracks;
}
	
// --- cBurnJob ---------------------------------------------------------------

cBurnJob::cBurnJob(void) {
	mTempDir = NULL;
	mTitle[0] = '\0';
	mDiskType = dtDVD;
	if (DVDdev == NULL)
		mStoreMode = smCreateISO;
	else if (ISOdir == NULL)
		mStoreMode = smBurnISO;
	else
		mStoreMode = smCreateBurn;
	mBurnMode = bmShrink;
	mProcess = NULL;
	marksSet=false;
}

cBurnJob::~cBurnJob() {
	delete mProcess;
	free(mTempDir);
}

void cBurnJob::Start(void) {
	if (mProcess) {
		esyslog("BURN: Process already running (This shouldn't happen!)");
		return;
	}

	switch (mDiskType) {
	case dtDVD:
		mProcess = new cBurnProcessDVD(this);
		break;

	case dtDVDnm:
		mProcess = new cBurnProcessDVD(this);
		break;

	case dtVDR:
		mProcess = new cBurnProcessVDR(this);
		break;

	default:
		break;
	};
}

void cBurnJob::Stop(void) {
	if (mProcess)
		mProcess->Stop();
}

void cBurnJob::Add(cBurnRecording *Object) {
	cBurnRecording *before = First();
	Object->SetNumber(Count());
	if (mTitle[0] == '\0')
		strncpy(mTitle, Object->Name(), JOBNAMELEN);
	while (before && before->Timestamp() < Object->Timestamp())
		before = Next(before);
	if (before)
		cListBase::Ins(Object, before);
	else
		cListBase::Add(Object);
}

void cBurnJob::SetTempDir(const char *TempDir) {
	for (cBurnRecording *rec = First(); rec; rec = Next(rec))
		rec->SetTempDir(TempDir);
	mTempDir = strdup(TempDir);
}

cBurnRecording *cBurnJob::GetByFileName(const char *FileName) {
	for (cBurnRecording *rec = First(); rec; rec = Next(rec)) {
		if (strcmp(rec->FileName(), FileName) == 0)
			return rec;
	}
	return NULL;
}

int cBurnJob::GetNextArchiveMark() {
	char* nextMarkerFileName = (char*)alloca(strlen(VideoDirectory) + strlen(NEXT_ARCHIVE_MARKER) + 1);
	if (!nextMarkerFileName) {
		return -1;
	}
	strcpy(nextMarkerFileName, VideoDirectory);
	char *end = nextMarkerFileName + strlen(VideoDirectory);
	strcpy(end, NEXT_ARCHIVE_MARKER);
	FILE* nextMarkerFile;
   
	int nextMark=1;
	if ((nextMarkerFile = fopen(nextMarkerFileName, "r")) != (FILE *)NULL) {
		char* nextDVDText=(char*)alloca(16);
		if (!nextDVDText) {
			fclose(nextMarkerFile);
			return -1;
		}
		if (fgets(nextDVDText, 16, nextMarkerFile)) {
			if ((end = strchr(nextDVDText, '\n'))) {
				*end = 0;
			}
		}
		fclose(nextMarkerFile);
                if (strlen(nextDVDText)) {
			int tmp=strtol(nextDVDText, &end, 10);
			if (*end==0) {
				nextMark=tmp;
			}
		}
	}
	return nextMark;
}

bool cBurnJob::CreateNextArchiveMark() {
	if(!BurnSetup.CreateArchiveMarks) {
		return true;
	}

	int nextMark=GetNextArchiveMark();
        if(nextMark<0) {
		return false;
	}

	nextMark++;

	char* nextMarkerFileName = (char*)alloca(strlen(VideoDirectory) + strlen(NEXT_ARCHIVE_MARKER) + 1);
	if (!nextMarkerFileName) {
		return false;
	}
	strcpy(nextMarkerFileName, VideoDirectory);
	char *end = nextMarkerFileName + strlen(VideoDirectory);
	strcpy(end, NEXT_ARCHIVE_MARKER);
	FILE* nextMarkerFile;
   
	if ((nextMarkerFile = fopen(nextMarkerFileName, "w")) != (FILE *)NULL) {
		int retVal=fprintf(nextMarkerFile, "%04d\n", nextMark);
		fclose(nextMarkerFile);
		if(retVal<0) {
			return false;
		}
	}
	return true;
}

bool cBurnJob::CreateArchiveMarks() {
	if(!BurnSetup.CreateArchiveMarks) {
		return true;
	}

	int nextMark=GetNextArchiveMark();
        if(nextMark<0) {
		return false;
	}

	cBurnRecording *rec;
	for (rec = First(); rec; rec = Next(rec)) {
		char* markerFileName=NULL;
		if(asprintf(&markerFileName, "%s/dvd.vdr", rec->FileName())<0) {
			return false;
		}
		if(access(markerFileName, F_OK)==0) {
			Interface->Confirm(tr("Recording already archived"), 200);
	                free(markerFileName);
			return false;
                }
                free(markerFileName);
	}

	for (rec = First(); rec; rec = Next(rec)) {
		char* markerFileName=NULL;
		if(asprintf(&markerFileName, "%s/dvd.vdr", rec->FileName())<0) {
			return false;
		}
                FILE* markerFile=NULL;
	        if ((markerFile = fopen(markerFileName, "w")) == (FILE *)NULL) {
			return false;
		}
		int retVal=fprintf(markerFile, "%04d\n", nextMark);
		marksSet=true;
		fclose(markerFile);
                free(markerFileName);
		if(retVal<0) {
			return false;
		}
	}
	return true;
}

bool cBurnJob::ClearArchiveMarks() {
	if(!marksSet) {
		return true;
	}

	cBurnRecording *rec;
	for (rec = First(); rec; rec = Next(rec)) {
		char* markerFileName=NULL;
		if(asprintf(&markerFileName, "%s/dvd.vdr", rec->FileName())<0) {
			return false;
		}
		unlink(markerFileName);
                free(markerFileName);
	}
	return true;
}

